--[[
Copyright 2008, 2009 João Cardoso
Sushi is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Sushi List.

Sushi is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Sushi is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Sushi. If not, see <http://www.gnu.org/licenses/>.
--]]

local Sushi = LibStub('Sushi-2.0')
local Container = Sushi:GetClass('Container')
local List, old = Sushi:NewWidgetClass('List', {
	version = 1
})

if not List then
	return
end


--[[ Events ]]--

function List:OnCreate()
	Container.OnCreate(self)
	self.layout = 'List'
	self.lines = {}
end

function List:OnRelease()
	self:SetChildren(nil)
	self:SetValue(nil)
	self:ClearLines()
end

function List:OnSizeChanged()
	self:UpdateLayout()
	
	local width = self:GetScrollChild():GetWidth() - 10
	for i, button in self:IterateChildren() do
		button:SetWidth(width)
	end
end


--[[ Lines ]]--

function List:AddLine(value, name)
	local name = value or name
	local button = self:CreateChild('ListButton')
	button:RegisterCallback('OnClick', self.OnLineClick, self, value, name)
	button:SetSelected(value == self:GetValue())
	button:SetText(name)
	
	self.lines[value] = button
	return button
end

function List:IterateLines()
	return pairs(self.lines)
end

function List:GetLine(value)
	return self.lines[value]
end

function List:ClearLines()
	return wipe(self.lines)
end

function List:OnLineClick(...)
	self:FireCallback('OnValueChanged', ...)
	self:FireCallback('OnLineClick', ...)
	self:FireCallback('OnUpdate')
end


--[[ Value ]]--

function List:SetValue(value)
	self.value = value
	self:UpdateChildren()
end

function List:GetValue()
	return self.value
end


--[[ Embedding ]]--

if not old then
	Container:Embed(List, 'OnCreate', 'OnRelease', 'OnSizeChanged', 'SetLayout', 'frameType', 'template', 'insets')
else
	List:UpdateEmbeds()
end